MODULE trcini_my_trc
   !!======================================================================
   !!                         ***  MODULE trcini_my_trc  ***
   !! TOP :   initialisation of the MY_TRC tracers
   !!======================================================================
   !! History :   2.0  !  2007-12  (C. Ethe, G. Madec) Original code
   !!----------------------------------------------------------------------
#if defined key_my_trc
   !!----------------------------------------------------------------------
   !!   'key_my_trc'                                               CFC tracers
   !!----------------------------------------------------------------------
   !! trc_ini_my_trc   : MY_TRC model initialisation
   !!----------------------------------------------------------------------
   USE par_trc         ! TOP parameters
   USE oce_trc
   USE trc
   USE trcsms_my_trc

   IMPLICIT NONE
   PRIVATE

   PUBLIC   trc_ini_my_trc   ! called by trcini.F90 module

   !!----------------------------------------------------------------------
   !! NEMO/TOP 3.3 , NEMO Consortium (2010)
   !! $Id$ 
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE trc_ini_my_trc
      !!----------------------------------------------------------------------
      !!                     ***  trc_ini_my_trc  ***  
      !!
      !! ** Purpose :   initialization for MY_TRC model
      !!
      !! ** Method  : - Read the namcfc namelist and check the parameter values
      !!----------------------------------------------------------------------
      INTEGER              ::  jn
      CHARACTER(len = 20)  ::  cltra


      jp_my_trc = 1
      ! assign an index in trc arrays for each prognostic variables
      DO jn = 1, jptra
        cltra = ctrcnm(jn) 
        IF( cltra == 'MYTRC'   .OR. cltra == 'mytrc'  )   jpmyt0 = jn 
        IF( cltra == 'MYTRC1'  .OR. cltra == 'mytrc1' )   jpmyt1 = jn 
        IF( cltra == 'MYTRC2'  .OR. cltra == 'mytrc2' )   jpmyt2 = jn 
      ENDDO
   
      IF( jp_my_trc == 1 ) THEN
        jp_myt0  = jpmyt0
      ELSE
        jp_myt0 = MIN( jpmyt0, jpmyt1, jpmyt2 ) 
      ENDIF
      jp_myt1 = jp_myt0 + jp_my_trc - 1


      IF( lwp ) THEN
        WRITE(numout,*) ''
        WRITE(numout,*) ' First index of MYTRC model in the passive tracer array   jp_myt0 = ', jp_myt0
        WRITE(numout,*) ' Last  index of MYTRC model in the passive tracer array   jp_myt1 = ', jp_myt1
        WRITE(numout,*) 
      ENDIF


      !                       ! Allocate MY_TRC arrays
      IF( trc_sms_my_trc_alloc() /= 0 )   CALL ctl_stop( 'STOP', 'trc_ini_my_trc: unable to allocate MY_TRC arrays' )

      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) ' trc_ini_my_trc: passive tracer unit vector'
      IF(lwp) WRITE(numout,*) ' To check conservation : '
      IF(lwp) WRITE(numout,*) '   1 - No sea-ice model '
      IF(lwp) WRITE(numout,*) '   2 - No runoff ' 
      IF(lwp) WRITE(numout,*) '   3 - precipitation and evaporation equal to 1 : E=P=1 ' 
      IF(lwp) WRITE(numout,*) ' ~~~~~~~~~~~~~~'
      
      IF( .NOT. ln_rsttr ) trn(:,:,:,jp_myt0:jp_myt1) = 1.
      !
   END SUBROUTINE trc_ini_my_trc

#else
   !!----------------------------------------------------------------------
   !!   Dummy module                                        No MY_TRC model
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE trc_ini_my_trc             ! Empty routine
   END SUBROUTINE trc_ini_my_trc
#endif

   !!======================================================================
END MODULE trcini_my_trc
